package gstd;
/**
 * Distribution includes a name, min value, max value, get and set methods
 *
 * @author <a href='mailto:mallett@cs.ualberta.ca'>Daniel Mallett</a>
 * @version %I%, %G%
 */
import java.util.*;

public abstract class Distribution implements Constants {
    protected String strName;
    private float[] fltMin;
    private float[] fltMax;
    private int intNumDimensions;

    public Distribution(String name, float[] min, float[] max) {
	intNumDimensions = Math.min(min.length,max.length);
	strName = name;
	fltMin = min;
	fltMax = max;
    }

    /**
     * return array of random numbers
     * dimension of array is defined by intNumDimensions
     *
     * @param random is the random number generator to use
     */
    public final float[] getRandom(Random random) {
	float[] retval = new float[intNumDimensions];
	for (int i = 0; i < intNumDimensions; i++) {
	    retval[i] = getRandom(random,fltMin[i],fltMax[i]);
	}
	return retval;
    }

    /**
     * return a random number from the distribution
     *
     * left up to the class implementor to define
     */
    public abstract float getRandom(Random random, float min, float max);

    /**
     * return the distribution's name
     */
    public String getName() {
	return strName;
    }
    
    public String getMinX() {
	if (fltMin == null) return "";
	return (new Float(fltMin[0])).toString();
    }
    public String getMinY() {
	try {
	    return (new Float(fltMin[1])).toString();
	} catch (ArrayIndexOutOfBoundsException e) {
	    return "";
	}
    }
    public String getMaxX() {
	if (fltMax == null) return "";
	return (new Float(fltMax[0])).toString();
    }
    public String getMaxY() {
	try {
	    return (new Float(fltMax[1])).toString();
	} catch (ArrayIndexOutOfBoundsException e) {
	    return "";
	}
    }
    public String getMean() {
	return "";
    }
    public String getSigma() {
	return "";
    }
    public String getSkew() {
	return "";
    }
    public String getType() {
	String type = "";
	if (getName().toLowerCase().charAt(0) == GAUSSIAN_DISTRIBUTION) {
	    type = "Gaussian";
	} else if (getName().toLowerCase().charAt(0) == RANDOM_DISTRIBUTION) {
	    type = "Uniform";
	} else if (getName().toLowerCase().charAt(0) == SKEWED_DISTRIBUTION) {
	    type = "Skewed";
	}
	return type;
    }
	
    public String[][] getSummary() {
	int numArgs = 1;
	if (!getMinX().equals("")) numArgs++;
	if (!getMinY().equals("")) numArgs++;
	if (!getMaxX().equals("")) numArgs++;
	if (!getMaxY().equals("")) numArgs++;
	if (!getMean().equals("")) numArgs++;
	if (!getSigma().equals("")) numArgs++;
	if (!getSkew().equals("")) numArgs++;

	String[][] info = new String[numArgs][2];

	int i = 0;
	info[i++] = new String[]{"Type",getType()};

	if (!getMinX().equals("")) {
	    info[i++] = new String[]{"Min X",getMinX()};
	}
	if (!getMinY().equals("")) {
	    info[i++] = new String[]{"Min Y",getMinY()};
	}
	if (!getMaxX().equals("")) {
	    info[i++] = new String[]{"Max X",getMaxX()};
	}
	if (!getMaxY().equals("")) {
	    info[i++] = new String[]{"Max Y",getMaxY()};
	}
	if (!getMean().equals("")) {
	    info[i++] = new String[]{"Mean",getMean()};
	    info[i++] = new String[]{"Sigma",getSigma()};
	}
	if (!getSkew().equals("")) {
	    info[i++] = new String[]{"Skew",getSkew()};
	}

	return info;
	
    }
}

