package gstd;
/**
 * Implements a Skewed Distribution
 * 
 * @author <a href='mailto:mallett@cs.ualberta.ca'>Daniel Mallett</a>
 * @version %I%, %G%
 */

import java.util.*;
import java.lang.*;
public class DistributionSkewed extends Distribution {
    private float fltSkew;

    public DistributionSkewed (float skew, float[] min, float[] max) {
	super("Skewed",min,max);
	fltSkew = skew;
    }
    /**
     * generates a random number taht follows Zipf distribution
     * and lies between min and max
     *
     * original code by Christos Faloutsos, 1995
     *
     * @param random random number generator
     * @param min minimum value of generated number
     * @param max maximum value of generated number
     */
    public float getRandom(Random random, float min, float max) {
	double HsubV = 0.0;
	int V = 100;

	/* calculate the V-th harmonic number HsubV. WARNING: V>1 */
	for (int i = 1; i <= V; i++) {
	    HsubV += 1.0/Math.pow(i,fltSkew);
	}
	
	double r = HsubV * 
	    (new DistributionRandom(new float[] {0},
				    new float[] {1})).getRandom(random)[0];
	double sum = 1.0;
	int i = 1;
	while (sum < r) {
	    i++;
	    sum += 1.0/Math.pow(i,fltSkew);
	}
	/* i follows Zipf distribution and lies between 1 and V */

	/* x flies between 0 and 1 and then between x1 and x2 */
	return (max - min) * (float)((i - 1.0) / (V - 1.0)) + min;
    }

    public String getName() {
	return strName + "_" + fltSkew;
    }
    public String getSkew() {
	return (new Float(fltSkew)).toString();
    }
}



