package gstd;
/**
 * Outputs the instances generated by GSTD_Abstract to an CSV file
 *
 * @author <a href='mailto:mallett@cs.ualberta.ca'>Daniel Mallett</a>
 * @version %I%, %G%
 */
import java.io.*;
import java.text.*;

public class GSTD_SpaceDelimited extends GSTD_Abstract implements Constants {
    /**
     * set the file name
     */
    public GSTD_SpaceDelimited(String fileName) {
	strFileName = fileName;
	extentNF = NumberFormat.getInstance();
	timeNF = NumberFormat.getInstance();
	extentNF.setMinimumFractionDigits(MIN_EXTENT_DIGITS);
	extentNF.setMaximumFractionDigits(MAX_EXTENT_DIGITS);
	timeNF.setMinimumFractionDigits(MIN_TIME_DIGITS);
	timeNF.setMaximumFractionDigits(MAX_TIME_DIGITS);
    }

    /**
     * create a new document with file name strFileName
     * and prepare it for writing
     */
    public void openOutput() throws IOException {
	fwOut = new FileWriter(strFileName);
    } 
    /**
     * close the output
     */
    public void closeOutput() throws IOException {
	fwOut.flush();
	fwOut.close();
    }
    /**
     * write to the document
     */
    public void outputInstance(Instance instance) throws IOException {
	String sp = " ";
	String output = instance.getID() + sp +
	    timeNF.format(instance.getTimeStamp().getTransactionTime()) + sp;

	float[] extents = instance.getSpaceStamp().getExtents();

	for (int i = 0; i < extents.length; i++) {
	    output += extentNF.format(extents[i]) + sp;
	}
	
	if (instance.isCurrent()) {
	    output += "1\n";
	} else {
	    output += "0\n";
	}

	fwOut.write(output);
    }
    
    FileWriter fwOut;    
    NumberFormat extentNF,timeNF;
}
