package gstd;
/**
 * Outputs the instances generated by GSTD_Abstract to an XML file
 *
 * @author <a href='mailto:mallett@cs.ualberta.ca'>Daniel Mallett</a>
 * @version %I%, %G%
 */
import java.io.*;
import java.text.*;

public class GSTD_XML extends GSTD_Abstract implements Constants {
    /**
     * set the file name
     */
    public GSTD_XML(String fileName) {
	strFileName = fileName;
	extentNF = NumberFormat.getInstance();
	timeNF = NumberFormat.getInstance();
	extentNF.setMinimumFractionDigits(MIN_EXTENT_DIGITS);
	extentNF.setMaximumFractionDigits(MAX_EXTENT_DIGITS);
	timeNF.setMinimumFractionDigits(MIN_TIME_DIGITS);
	timeNF.setMaximumFractionDigits(MAX_TIME_DIGITS);
    }

    /**
     * create a new XML document with file name strFileName
     * and prepare it for writing
     */
    public void openOutput() throws IOException {
	if (fwOut == null) {
	    fwOut = new FileWriter(strFileName);
	}
	fwOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	fwOut.write("<!DOCTYPE gstd SYSTEM \"../../gstd.dtd\">\n");
	fwOut.write("<gstd type=\"" + chrObjectType + "\">\n");
    } 
    /**
     * close the XML output
     */
    public void closeOutput() throws IOException {
	fwOut.write("</gstd>\n");
	fwOut.flush();
	fwOut.close();
    }
    /**
     * write to the XML document
     */
    public void outputInstance(Instance instance) throws IOException {
	String output = "<instance " +
	    "valid=\"" + instance.isCurrent() + "\" " +
	    "id=\"" + instance.getID() + "\" " +
	    "time=\"" + timeNF.format(instance.getTimeStamp().getTransactionTime()) + "\"" +
	    ">\n";
	float[] extents = instance.getSpaceStamp().getExtents();

	for (int i = 0; i < extents.length; i++) {
	    char point = 'x';
	    if ((i%2) != 0) 
		point = 'y';

	    //don't output the top left x y for points (it's redundant)
	    if (chrObjectType == POINT_OBJECT_TYPE && i > 1) break;

	    output += "\t<" + point + ">" + extentNF.format(extents[i]) +
		"</" + point + ">\n";
	}
	output += "</instance>\n";
	fwOut.write(output);
    }

    protected FileWriter fwOut;
    NumberFormat extentNF,timeNF;
}

