package gstd;
/**
 * Space stamp is center point and extents
 *
 * @author <a href='mailto:mallett@cs.ualberta.ca'>Daniel Mallett</a>
 * @version %I%, %G%
 */
public class SpaceStamp implements Constants, Cloneable {  
    float[] center;
    float[] extent;
    char    chrObjectType;

    public SpaceStamp(char objectType, float[] c, float[] ext) {
	chrObjectType = objectType;

	center = c;

	if (chrObjectType == RECTANGLE_OBJECT_TYPE) {
	    //rectangle 
	    //initial extent follows initial density value
	    //area = density / num snap shots
	    extent = ext;

	} else {
	    //point 
	    //the extent is initialize to 0 by the constructor
	    extent = new float[c.length];
	}

    }

    public final String toString() {
	String retval = "";
	float[] extents = getExtents();
	for (int i = 0; i < extents.length; i++) {
	    retval += extents[i];
	    if (i < extents.length - 1) 
		retval += ",";
	}
	return retval;
    }

    /**
     * return the spatial extents in the form x1,y1,x2,y2
     */
    public final float[] getExtents() {
	float[] retval = new float[NDIMS*2];

	for (int i = 0; i < NDIMS; i++) {
	    retval[i] = center[i] - (extent[i] / (float)2.0);
	}
	for (int i = NDIMS; i < NDIMS*2; i++) {
	    retval[i] = center[i-NDIMS] + (extent[i-NDIMS] / (float)2.0);
	}
	return retval;
    }

    public float[] getCenter() {
	return center;
    }
    public float[] getExtent() {
	return extent;
    }
    public void setCenter(float[] cet) {
	center = cet;
    }
    public void setExtent(float[] ext) {
	extent = ext;
    }
    public char getObjectType() {
	return chrObjectType;
    }
    /**
     * update the space stamp values by the delta 
     *
     * @param deltaCenter change in center
     * @param deltaExt change in extent
     */
    public final void update(float[] deltaCenter, float[] deltaExt) {
	try {
	    for (int i = 0; ; i++) {
		center[i] += deltaCenter[i];
		
		if (chrObjectType == RECTANGLE_OBJECT_TYPE) {
		    extent[i] += deltaExt[i];
		    
		    // necessary adjustment for rectangles
		    if (extent[i] < 0.)
			extent[i] = (float)0.;

		}
	    }
	} catch (ArrayIndexOutOfBoundsException e) {}
    }


    /**
     * true if the point or rectangle is wholy within the unit square
     */
    public boolean isInUnitSquare() {
	for (int i = 0; i < NDIMS; i++) {
	    if ( (center[i] - (extent[i] / 2.) < 0.) ||
		 (center[i] + (extent[i] / 2.) > 1.) )
		return false;
	}
	return true;
    }

    /**
     * perform the radar, toroid or adjustment approach
     *
     * return true if the point is still valid (ie. in the unit sphere)
     * 
     */
    public boolean adjust(char chrApproach) {
	switch (chrApproach) {
	case RADAR_APPROACH:
	    // radar: no adjustment - set 'correct' flag
	    return isInUnitSquare();
	    
	case TOROID_APPROACH:
	    for (int i = 0; i < NDIMS; i++) {
		if ( center[i] - (extent[i] / 2.) < 0.)
		    center[i] = 1 + center[i];
		if ( center[i] + (extent[i] / 2.) > 1.)
		    center[i] = center[i] - (int)center[i];
	    }
	    break;
	    
	case ADJUST_APPROACH:
	    for (int i = 0; i < NDIMS; i++) {
		if ( center[i] - (extent[i] / 2.) < 0.)
		    center[i] = (float)(extent[i] / 2.);
		if ( center[i] + (extent[i] / 2.) > 1.)
		    center[i] = (float)(1. - extent[i] / 2.);
	    }
	    
	    break;
	}
	//toroid and adjustment should always be valid
	return isInUnitSquare();
    }

    public Object clone() {
	float[] c = new float[center.length];
	float[] e = new float[extent.length];
	for (int i = 0; i < center.length; i++) {
	    c[i] = center[i];
	    e[i] = extent[i];
	}
	return new SpaceStamp(chrObjectType,c,e);
    }

    /**
     * return true if the space stamp ssIntersect intersects
     * with this space stamp
     */
    public boolean intersects(SpaceStamp ssIntersect) {
	if (chrObjectType == RECTANGLE_OBJECT_TYPE) {

	    //if (ssIntersect.getObjectType() == RECTANGLE_OBJECT_TYPE) {
		//check if rectangle intersects this rectangle
		float[] thisExtent = getExtents();
		float[] intersectExtent = ssIntersect.getExtents();
		if ( (thisExtent[0] < intersectExtent[2]) &&
		     (intersectExtent[0] < thisExtent[2]) &&
		     (thisExtent[1] < intersectExtent[3]) &&
		     (intersectExtent[1] < thisExtent[3]))
		    return true;
		return false;
		//} 

	    //check if point in this rectangle
		/*
	    float[] intersectCenter = ssIntersect.getCenter();
	    for (int i = 0; i < NDIMS; i++) {
		if ( (center[i] - (extent[i] / 2.) >= intersectCenter[i]) ||
		     (center[i] + (extent[i] / 2.) <= intersectCenter[i]))
		    return false;
	    }
	    return true;
		*/
	} 

	//this is a point - see if this intersects ssIntersect
	if (ssIntersect.getObjectType() == RECTANGLE_OBJECT_TYPE) {
	    return ssIntersect.intersects(this);
	} 
	//two points can't intersect
	return false;
    }
}
